function YEst = KRR(xind, yind, DM, whichTimb, s, lambda)
% Do Kernel Ridge Regression.
%
%% "Y" = training data
% ynm = y note matrix (unquantized) (global)
% ynmq = y note matrix, quantized (global)
% yind = which notes of ynm and ynmq to consider part of training set
%% "X" = new input
% global xnmq = x note matrix, quantized
% xind = which notes of xnmq to consider part of the test set
% xnm is unknown, the output of our algorithm
%
% whichTimb: only use this timbre (***set to 0 to use all of the timbres at
% the same time***)
%
% DM is the distance matrix whose rows are elements of xnmq and
% columns are elements of ynmq
%
% For example: KRR([1:100], [101:500], 0, DM)

global xnmq ynmq ynm 

if nargin<6
  lambda = 1;
end

if nargin<5
  % "s" means "sigma squared"
  s = 1000; 
end

doplot = 1;
biasTerm = 0;  % This may not be a good idea; zero should be safe.

%% "Y" = training data
%% "X" = new input
%% New names for old code:
testInd = xind;
trainInd = yind;


% Here's where we cheat, knowing the right answers, just for
% comparison's sake
Ytest = ynm(testInd,1) - ynmq(testInd,1);


disp(['Training on ' num2str(length(trainInd)) ' notes, testing on ' num2str(length(testInd)) ...
    ' notes.'])

% Single-timbre mode

if and(nargin >= 4, whichTimb ~= 0)
  if 0
    timbInd = find(nm(:,4) == whichTimb);
    nm = nm(timbInd,:);
    nmq = nmq(timbInd,:);
  else
   error('Single timbre mode not implemented');
  end
end


phiTphi = exp(-((DM+biasTerm).^2)/2/s);

k = phiTphi(trainInd, trainInd);
Ytrain = ynm(trainInd,1) - ynmq(trainInd,1);

indices_used = unique([trainInd testInd]);
phiTphi_used = phiTphi(indices_used, indices_used);
e = eig(phiTphi_used);
if (lambda < min(e))
  lambda = min(e) + 0.1;   % this 0.1 is somewhat arbitrary for the moment ...
  ['WARNING: had to increase lambda to ' num2str(lambda) ' because the eigs were too small!']
  disp('Cancelling - your lambda sucks.');
  YEst = ones(length(testInd), 1) * -999999;
  return;
end

YEst = zeros(length(testInd),1);
YEstNN = zeros(length(testInd),1);

    
for count = 1:length(testInd)
  if mod(count,100) == 0
    disp(['Working on test set element ' num2str(count)]);
  end

  i=testInd(count);

  kappa = phiTphi(trainInd, i);

  if (sum(isinf(kappa)))
      disp(['(TESTING) Distance between ' num2str(i) ' and test set' ...
		    ' is ' num2str(kappa)]);
  end
    
  % Kernel Ridge Regression using our pseudokernel

  tmp = ((k + lambda*(eye(length(trainInd)))) \ kappa);
  %stem(tmp)
  YEst(count)= Ytrain' * tmp;
  %pause
  
  % 1-NN
  [val, ind] = max(kappa);
  YEstNN(count) = Ytrain(ind);
  
%  disp(['Yest = ' num2str(YEst(count)) ' vs Ytest = ' num2str(Ytest(count)) ...
%        'vs YEstNN = '  num2str(YEstNN(count))])

  % disp([num2str(round(100*(count))/length(testInd)) '% done'])
end

if doplot
    figure(5)
    subplot(2,1,1)
    stem(Ytest)
    a = axis;
    title('Test set')
    subplot(2,1,2)
    stem(Ytrain)

    axis(a)
    title('Training set')


    figure(7)
    hold off
    stem(Ytest)
    hold on
    stem(YEst,'r')


    errVarNN = var(Ytest - YEstNN);
    errVar = var(Ytest - YEst);
    mtVar = var(Ytest);
    
    disp(['errVarNN: ' num2str(errVarNN) ', errVar: ' num2str(errVar) ...
	  ', mtVar: ' num2str(mtVar)]);

    absErrNN = mean(abs(Ytest - YEstNN));
    absErr = mean(abs(Ytest - YEst));
    absMt = mean(abs(Ytest));

    
    disp(['absErrNN: ' num2str(errVarNN) ', absErr: ' num2str(absErr) ...
	  ', absMt: ' num2str(absMt)]);

    bRatio = length(find(abs(Ytest-YEst) < abs(Ytest-YEstNN))) / length(Ytest)
end
