function [xnm matches distances] = apply_kNN(xind, yind, k, tau, DM)
%% "Y" = training data
% ynm = y note matrix (unquantized) (global)
% ynmq = y note matrix, quantized (global)
% yind = which notes of ynm and ynmq to consider part of training set
%% "X" = new input
% global xnmq = x note matrix, quantized
% xind = which notes of xnmq to consider part of the test set
% xnm is unknown, the output of our algorithm


if (nargin < 5)
  have_DM = 0;
else
  have_DM = 1;
end

if (nargin < 4)
   tao = 5;
end

if (nargin < 3)
     k = 1;
end

disp(['k-nearest-neighbors, k=' num2str(k) ', tau = ' num2str(tau)]);

% Make sure they're row vectors so we can use "for"
xind = xind(:)';
yind = yind(:)';


lenY = length(yind);

if or(k == 0, k > lenY)
     k = lenY;
     disp(['Setting k to ' num2str(k) ', the size of the training set']);
end

global xnmq ynmq ynm;

% Check that indices are valid
xnmq(xind,:);
ynmq(yind,:);


xlen = max(xind);
matches = zeros(xlen, k);
distances = zeros(xlen, k);

% For now, output sequence is same as quantized input sequence
% (to get timbre, loudness, etc.); we'll overwrite the first column
% with our algorithm's' microtimings
xnm = xnmq;





progress = 0;
for xi = xind
    progress = progress + 1;
    if mod(progress, 1000) == 0
       disp(['working on note ' num2str(progress)]);
    end

    % Find this note's distance to every note in the training set
    if have_DM
      mydistances = DM(xi,  yind);
    else
      mydistances = zeros(1, lenY);
      for i = 1:lenY;
        yi = yind(i);
	% pair = [xi yi]
	mydistances(i) = notedistance(xi, yi);
      end
    end

    [sorted_distances, sorted_indices] = sort(mydistances);
    if sorted_distances(1) == inf
       error('Every distance was infinity!');
    end

    % k Nearest Neighbors
    closest_ys = yind(sorted_indices([1:k]));
    best_distances = sorted_distances([1:k]);

    matches(xi,:) = closest_ys;
    distances(xi,:) = best_distances;

    unscaled_weights = exp(-(best_distances.^2) / (tau^2));
    if max(unscaled_weights) == 0
      error(['tau of ' num2str(tau) ' is so small that all weights' ...
		    ' are zero.']);
    end

    weights = unscaled_weights .* (1/sum(unscaled_weights));    
    found_devs = ynm(closest_ys,1) - ynmq(closest_ys,1);

%    sizeweights = size(weights)
%    sizefound_devs = size(found_devs)

    % weighted average of knn deviations
    output_dev = weights * found_devs;

    xnm(xi,1) = xnmq(xi,1) + output_dev;
end

matches=matches(xind);
distances=distances(xind);

disp('done with kNN');
