function [errMetric, wts] = doGP(Xtrain, Ytrain, Xtest, Ytest, its, prevWts)
% errMetric = [mtVar testErrVar trainErr meanAbsMt testMeanAbsErr nnErr snErr];

[xm, xn] = size(Xtrain);

if (nargin < 5)
    its = 40;
end

if (nargin < 6)
    wtsGiven = 0;
else
    wtsGiven = 1;
end

W = xn + 2;
%if (nargin < 5) % need to estimate the weights
totStarts = 1;  % It seems to always converge to similar solutions, so just run once
wts = zeros(W,totStarts);
fvals = zeros(totStarts,1);
for i=1:totStarts,
    numFailures = 0;
    success = 0;
    while and(numFailures < 5, not(success))
        try
            if ~wtsGiven
                prevWts = rand(W,1) - 0.5*ones(W,1);   % start from random
            end
            % In case its == 0
            wtsout = prevWts;
            fWts = zeros(size(wtsout));
            itsout = 0;
            
            if (its ~= 0)
                [wtsout, fWts, itsout] = minimize(prevWts, 'gp01lik', its, Xtrain, Ytrain);
            end
            success = 1;
        catch
            % since minimize fails occassionally
            numFailures = numFailures + 1;
            msg = lasterror;

            disp(msg)
        end
    end
    wts(:,i) = wtsout;
    fvals(i) = min(fWts);      % minimal values achieved by the optimization
    itsout(i) = itsout;
end
disp('exp(converged weights) was:')
exp(wts)
%end

% Try Gaussian Processes
[mu, S2, deriv, S2deriv] = gp01pred(wts, Xtrain, Ytrain, Xtest);

% Check the training error
[trainMu, dummy] = gp01pred(wts, Xtrain, Ytrain, Xtrain);
trainErr = var(Ytrain-trainMu);


% Comparison with nearest neighbor (don't even plot this one it's so bad)
N = length(Ytest);
M = length(Ytrain);
nn = zeros(N,1);
for n=1:N,
    dist = inf;
    ind = 0;
    for m=1:M,
        cmpDist = norm(Xtest(n,:) - Xtrain(m,:));
        if (cmpDist < dist)
            dist = cmpDist;
            ind = m;
        end
    end
    nn(n) = Ytrain(ind);
end


% Comparison with scaled nearest neighbor
N = length(Ytest);
M = length(Ytrain);
snn = zeros(N,1);
swts = sqrt(exp(wts(1:W-2)))';
for n=1:N,
    dist = inf;
    ind = 0;
    for m=1:M,
        cmpDist = norm(swts .* (Xtest(n,:) - Xtrain(m,:)));
        if (cmpDist < dist)
            dist = cmpDist;
            ind = m;
        end
    end
    snn(n) = Ytrain(ind);
end

muVar = var(mu);
mtVar = var(Ytest);
testErrVar = var(Ytest-mu);
mtVar = var(Ytest);
meanAbsMt = mean(abs(Ytest));
testMeanAbsErr = mean(abs(Ytest-mu));
nnErr = var(Ytest-nn);
snErr = var(Ytest-snn);

if (0)
    figure(1)
    hold off
    stem(Ytest)
    hold on
    stem(mu,'r')
    stem(nn,'g')
    stem(snn,'k-.')
    legend('Actual','GP','NN','SNN')
end

figure(2)
hold off
stem(Ytest,'r')
hold on
errorbar(1:length(mu),mu,mu-sqrt(S2),mu+sqrt(S2))
legend('Actual','GP')




errMetric = [mtVar testErrVar trainErr meanAbsMt testMeanAbsErr nnErr snErr];
