% simple PCA test
function [whichInd, Vout, mus, stds, Xproj] = doPCA(X, numBases, icaInstead, flag)

% Note: Careful about messing with nargin!!!
% This is actually what we used in the paper for the GP tests.

[M,N] = size(X);

sm = std(X);
whichInd = find(sm ~= 0);
NN = length(whichInd);

disp(['doPCA: ' num2str(round(100*length(whichInd)/N)) '% of the columns had nonzero variance.'])

Xsm = X(:,whichInd);
mus = mean(Xsm);

Xzm = Xsm - ones(M,1) * mus;

stds = std(Xzm);
Xzms = Xzm ./ (ones(M,1) * stds);
[Mshrunk, Nshrunk] = size(Xzms);
if (Mshrunk <= Nshrunk)
    [U, S, V] = svd(Xzms);         % there may not really be enough
    % data in this case ...
else
    [U, S, V] = svd(Xzms, 'econ');   % but for lots of data, econ is
    % MUCH faster
end
% Select the eigenvectors with the largest eigenvalues
Vout = V(1:numBases,:)';

% Carry out the projection to a smaller space
Xproj = Xzms * Vout;


if nargin > 2
    if icaInstead==1
        % Transposes are required because the ICA
        % people think the other way around ...
        [icasig, A, W] = fastica(Xzms', 'lastEig', 20, 'numOfIC', numBases);  % was 7
        icasig = icasig';
        % The "whitening matrix" W is such that Xzms * W' == icasig
    end
end


% Plot the results if desired
if (nargin > 3)
    figure(2)
    hold off
    bar(diag(S))
    title('Singular values')
    ylabel('Magnitude')
    xlabel('Index of the singular value')


    [dummy, timbres] = timbre_name(0, 0);
    T = length(timbres);
    B = 17; % length of detected beat vector


    figure(3)
    hold off
    mn = mean(mean(Vout));
    rnge = max(max(Vout)) - min(min(Vout));
    for i=1:numBases,
        v = zeros(B*T,1);
        v(whichInd) = V(i,:);

        explTime = zeros(B*T,1);

        for b=1:B,
            for t=1:T,
                explTime(t + (b-1)*T) = v(t + (b-1)*T);
            end
        end

        L = 256;
        %G = zeros(L,1);
        %Bl = max(zeros(L,1), linspace(1, -1, L)');
        %R = max(zeros(L,1), linspace(-1, 1, L)');

        % (this one has a white background)
        Bl = [ones(L/2,1); linspace(1,0,L/2)'];
        R = [linspace(0,1,L/2)'; ones(L/2,1)];
        G = [linspace(0,1,L/2)'; linspace(1,0,L/2)'];

        colormap([R G Bl])

        subplot(1,numBases,i)
        eigRhythm = reshape(explTime,T,B);
        image((eigRhythm-mn)/rnge*255 + 128)

        xlabel('Time')

        if (i==1)
            ylabel('Timbre')
        end

        if (i==3)
            title('EIGENRHYTHMS FROM PCA')
        end
        axis xy
    end


    figure(4)
    for i=1:numBases,
        subplot(numBases,1,i)
        plot(Xproj(:,i))
        if (i==1)
            title('Principal components')
        end
    end

    if nargin > 2
        if icaInstead==1
            figure(5)

            hold off
            mn = mean(mean(W));
            rnge = max(max(W)) - min(min(W));
            for i=1:numBases,
                v = zeros(B*T,1);
                v(whichInd) = W(i,:);

                explTime = zeros(B*T,1);

                for b=1:B,
                    for t=1:T,
                        explTime(t + (b-1)*T) = v(t + (b-1)*T);
                    end
                end

                L = 256;
                %G = zeros(L,1);
                %Bl = max(zeros(L,1), linspace(1, -1, L)');
                %R = max(zeros(L,1), linspace(-1, 1, L)');

                % (this one has a white background)
                Bl = [ones(L/2,1); linspace(1,0,L/2)'];
                R = [linspace(0,1,L/2)'; ones(L/2,1)];
                G = [linspace(0,1,L/2)'; linspace(1,0,L/2)'];

                colormap([R G Bl])

                subplot(1,numBases,i)
                eigRhythm = reshape(explTime,T,B);
                image((eigRhythm-mn)/rnge*255 + 128)

                xlabel('Time')

                if (i==1)
                    ylabel('Timbre')
                end

                if (i==3)
                    title('INDIRHYTHMS FROM ICA')
                end
                axis xy
            end



            figure(6)
            for i=1:numBases,
                subplot(numBases,1,i)
                plot(icasig(:,i))
                if (i==1)
                    title('Independent components')
                end
            end
        end
    end


    drawnow
end


if nargin > 2
    % Output the ICA data instead if requested
    if icaInstead==1
        Xproj = icasig;
        Vout = W';
    end
end



dbgging = 0;
if (dbgging)
    % Make sure that projecting the test data will work ...
    sm = Xproj - Xzms*Vout;
    totError = sum(sum(sm.^2))
    if (totError > 1e-10)
        error('Something is fishy in doPCA.m')
    end
end
