function [timbs, timbMt, mtMat] = estimateMultiTimbGrooveTemplates(nm, nmq, trainInd, beats)
% Estimates and returns a multi-timbreal groove template according to the
% given notematrices and the indices designated for training.
%
% timbs is a vector indicating which timbres are included in the rhythm.


% Training
timbs = unique(nmq(:,4));
T = length(timbs);
B = length(beats);
timbMt = zeros(T,1);
mtMat = zeros(B,T);
occsMat = zeros(B,T);

nmqtrain = nmq(trainInd,:);
nmtrain = nm(trainInd,:);
[M,N] = size(nmqtrain);
for m=1:M,
    timbInd = find(timbs == nmqtrain(m,4));
    beatInd = find(beats == mod(nmqtrain(m,1),4));

    mtMat(beatInd, timbInd) = mtMat(beatInd, timbInd) + nmtrain(m,1) - nmqtrain(m,1);
    occsMat(beatInd, timbInd) = occsMat(beatInd, timbInd) + 1;
end

% Normalize the sum of the microtimings by the number of occurances
[indm, indn] = find(not(occsMat==0));
for z=1:length(indm),
    mtMat(indm(z), indn(z)) = mtMat(indm(z), indn(z)) / occsMat(indm(z), indn(z));
end

