% function handle_cyclical_reggae.m
%
% Join together related segments into note matrices ensuring that segments
% start on multiples of 4.  Provide an extra beat at the beginning and the
% end of each segment to ensure that notedistance.m has enough information.
%
% However, to avoid counting the wrap-arounds as valid training data,
% store the resulting useful indices while segmenting them into groups of
% 5.
%

load ourdata/Reggae/Quantized/NoteMatrices.mat

n = length(fnames);
allinstruments = [];
totalnotes = 0;

% First find all instruments used
for i = 1:n
    nm = nm_cell{i};
    instruments = unique(nm(:,4));
    allinstruments = unique([allinstruments ; instruments]);
end

bigtotalperinstrument = [allinstruments , zeros(size(allinstruments))];

for i = 1:n
    f = fnames{i};
    nm = nm_cell{i};    
    total = size(nm, 1);
    instruments = unique(nm(:,4));
    numinsts = length(instruments);
    totalperinstrument = zeros(numinsts, 2);

    totalperinstrument(:,1) = instruments;
    for i = 1:numinsts
        totalperinstrument(i,2) = sum(nm(:,4) == instruments(i));
    end

    disp(['File ' f ' has ' num2str(total) ' notes in ' ...
          num2str(numinsts) ' instruments']);
%    totalperinstrument

     for j = 1:numinsts
	 this = totalperinstrument(j, 1);
	 ind = find(allinstruments(:,1)==this);
	 bigtotalperinstrument(ind, 2) = bigtotalperinstrument(ind, 2) + ...
	    totalperinstrument(j,2);
     end


    totalnotes = totalnotes + total;
end

disp(['Entire dataset has ' num2str(totalnotes) ' notes, like this:']);

bigtotalperinstrument


windowLen = 1;  % look for 1-beat wrap-arounds
fignum = 1;
nm_cell_out = {};
nmq_cell_out = {};
% Not grouping 120MIX
reggaerhythmnames = {'eightreg','lvrrock','skapatt'};
reggaerhythm_seqnames = {};
reggaerhythm_totInd = {};
rNum = 1;
for theRhythm = {'80REG', 'LVRK', 'SKAPATT'},
    whichnames = strfind(fnames,theRhythm{1});
    L = length(whichnames);

    nmout = [];
    nmqout = [];
    currentBeat = 4;    % start ahead so that we have some leeway
    cnt = 1;
    theseNames = {};
    theseInd = {};
    totInd = [];
    for i=1:L,
        if whichnames{i},
            theseNames{cnt} = fnames{i};
                
            nmqdelay = nmq_cell{i};
            nmdelay = nm_cell{i};
            % The tempi should be all the same or there will be problems!
            T = gettempo(nmqdelay);
            [M,N] = size(nmqdelay);
                      
            lastLocalBeat = 4 * ceil(nmqdelay(M,1) / 4);
            dists = lastLocalBeat - nmqdelay(:,1);
            % boundary case: throw out onsets on last beat if
            % it's a multiple of four ...
            prefixInd = find(and(dists <= 1, dists > 0));
            nmqprefix = nmqdelay(prefixInd,:);
            nmprefix = nmdelay(prefixInd,:);
            MM = length(prefixInd);
            
            nmqprefix(:,1) = nmqprefix(:,1) + (currentBeat-lastLocalBeat) * ones(MM,1);
            nmqprefix(:,6) = nmqprefix(:,6) + (currentBeat-lastLocalBeat) * ones(MM,1) * 60 / T;
            nmqout = [nmqout; nmqprefix];
            nmprefix(:,1) = nmprefix(:,1) + (currentBeat-lastLocalBeat) * ones(MM,1);
            nmprefix(:,6) = nmprefix(:,6) + (currentBeat-lastLocalBeat) * ones(MM,1) * 60 / T;
            nmout = [nmout; nmprefix];
            
            % Handle the meat of the sequence itself
            nmqdelay(:,1) = nmqdelay(:,1) + currentBeat * ones(M,1);
            nmqdelay(:,6) = nmqdelay(:,6) + currentBeat * ones(M,1) * 60 / T;
            nmqout = [nmqout; nmqdelay];
            nmdelay(:,1) = nmdelay(:,1) + currentBeat * ones(M,1);
            nmdelay(:,6) = nmdelay(:,6) + currentBeat * ones(M,1) * 60 / T;
            [pInd, dummy] = size(nmout);
            nmout = [nmout; nmdelay];
            [nInd, dummy] = size(nmout);
            theseInd{cnt} = [pInd+1:nInd];
            totInd = [totInd pInd+1:nInd];    % keep track in an array too for partitioning
            
            % boundary case: this one is easier because we will also
            % be taking the beat at zero along
            nmqdelaybeforeshift = nmq_cell{i};
            nmdelaybeforeshift = nm_cell{i};
            postfixInd = find(nmqdelaybeforeshift(:,1) <= 1);
            nmqpostfix = nmqdelaybeforeshift(postfixInd,:);
            nmpostfix = nmdelaybeforeshift(postfixInd,:);
            MM = length(postfixInd);
            nmqpostfix(:,1) = nmqpostfix(:,1) + (currentBeat + lastLocalBeat) * ones(MM,1);
            nmqpostfix(:,6) = nmqpostfix(:,6) + (currentBeat + lastLocalBeat) * ones(MM,1) * 60 / T;
            nmqout = [nmqout; nmqpostfix];            
            nmpostfix(:,1) = nmpostfix(:,1) + (currentBeat + lastLocalBeat) * ones(MM,1);
            nmpostfix(:,6) = nmpostfix(:,6) + (currentBeat + lastLocalBeat) * ones(MM,1) * 60 / T;
            nmout = [nmout; nmpostfix];    
            
            % Update the counters
            currentBeat = 4 * ceil(nmqout(length(nmqout),1) / 4);
            cnt = cnt + 1;
        end
    end
    reggaerhythm_seqnames{rNum} = theseNames;
    reggaerhythm_ind{rNum} = theseInd;
    reggaerhythm_totInd{rNum} = totInd';
    nm_cell_out{rNum} = nmout;
    nmq_cell_out{rNum} = nmqout;
    rNum = rNum + 1;
    
    writemidi(nmqout, ['ourdata/Reggae/Quantized/Cyclical/' theRhythm{1} '-Q.MID'], 10000, gettempo(nmqout));
    writemidi(nmqout, ['ourdata/Reggae/Quantized/Cyclical/' theRhythm{1} '.MID'], 10000, gettempo(nmqout));
    
    if (0)
    figure(fignum)
    fignum = fignum + 1;
    pianoroll(nmout)
    pianoroll(nmqout,'hold','b')
    title(['Cyclical representation of ' theRhythm{1}])
    end
end
nmq_reg_cell = nmq_cell_out;
nm_reg_cell = nm_cell_out;

save('~/Documents/cs229/ourdata/Reggae/Quantized/NoteMatrices.cyclical.mat', 'reggaerhythm_seqnames', 'reggaerhythm_ind', 'reggaerhythm_totInd', 'nm_reg_cell', 'nmq_reg_cell','reggaerhythmnames');