function [F] = make_F(nmq,flg)
% Formulate the other features in the vector F
%
% Currently:
%
%    [F_1,2] = [0,0]: if onset is within first beat of the bar
%    [F_1,2] = [0,1]: if onset is within third beat of the bar
%    [F_1,2] = [1,0]: if onset is within second beat of the bar
%    [F_1,2] = [1,1]: if onset is within fourth beat of the bar
%
%    [F_3,4] = [0,0]: if onset is within first 1/16th note of the beat
%    [F_3,4] = [0,1]: if onset is within third 1/16th note of the beat
%    [F_3,4] = [1,0]: if onset is within second 1/16th note of the beat
%    [F_3,4] = [1,1]: if onset is within fourth 1/16th note of the beat
%
%    [F_5] = 1 iff onset is on a 1/32nd note
%
%    [F_6] = timbreal height (as returned by timbre_name)
%
%    [F_7] = unusualness measure of the timbre (only used if flg==1)
%
%    [F_7] = velocity of "this" onset (only used if flg==2)


global unusualness totTimb   % from load_data

if nargin < 2
    flg = 0;
end


% Vector mapping of indices 1:4
vMap = {[0 0], [0 1], [1 0], [1 1]};

% Using this many dimensions total
% (Except that one may be added later for velocity if requested ...)
N = 6 + (flg==1);

% Make X
[M,dummy] = size(nmq);
F = zeros(M, N);

for i=1:M,
    beat = nmq(i,1);
    notenumber = nmq(i,4);
    
    quarterNote = floor(beat);
    F(i,1:2) = vMap{mod(quarterNote,4)+1};
    
    sixteenthNote = floor((beat - quarterNote) * 4) / 4;
    F(i,3:4) = vMap{4*sixteenthNote+1};
    
    thrtyscndNote = floor((beat - quarterNote - sixteenthNote) * 8) / 8;
    F(i,5) = thrtyscndNote * 8;   % these should be mostly 0's with a few 1's here and there
    
    [name, timbHeight] = timbre_name(notenumber);
    F(i,6) = timbHeight;
    
    if (flg==1)
        ind = find(totTimb == notenumber);
        if length(ind) > 0
            F(i,7) = unusualness(ind(1));
        end
    end
end


% Add a column for velocity
if (flg==2)
    F = [F nmq(:,5)];
end
