function [X,Y] = make_X(nm, nmq, timbre,beatLocs)
% Makes the vector-formulated X matrix from the given note matrix
% Currently works only in the one-timbre version (3rd arg is MIDI note #)

% Assumes beats and beatLocations exist (from having run load_data)
% Output vector elements for each note:
%  [[2D_Beat_Space_Position] ... 
%   [whether this drum plays on each of the eight 32nd notes before and after]]
% That's 2 + 8 + 8 = 18 columns

if (size(nm) ~= size(nmq))
   error('nm and nmq must be the same size!');
end

global beats;
[dummy,BL] = size(beatLocs);


if (~exist('beats'))
   error('Global var "beats" must exist');
end

ind = find(nm(:,4) == timbre);

if isempty(ind)
   error(['No notes of nm have timbre ' num2str(timbre)]);
end

% Portions of input note matrices for just the timbre we care about:
nmt = nm(ind, :);
nmqt = nmq(ind, :);


% Make Y
Y = nmt(:,1) - nmqt(:,1);

% Make X
n = size(nmt, 1);
X = zeros(n, 16+BL);
sizeX = size(X);

noteBeats = nmqt(:, 1);

for i=1:n
    beat = noteBeats(i);
    modbeat = mod(beat, 4);
    index = (modbeat*8) + 1;
    X(i,1:BL) = beatLocs(index,:);

    beatsbefore = [beat-1 : 1/8 : beat - 1/8];
    beatsafter = [beat+1/8 : 1/8 : beat+1];

    for j=1:8
	if ~isempty(find(noteBeats == beatsbefore(j)))
	   X(i, BL+j) = 1;
        end
	if ~isempty(find(noteBeats == beatsafter(j)))
	   X(i, BL+8+j) = 1;
        end
    end
end
