function [X,Y] = make_large_X(nm, nmq, flg)
% Makes the vector-formulated X matrix from the given note matrix
%
% This does not change the order of the Y's, so you can generate
% your own Y if you like.
%
% Considers whether a given drum drum plays on this beat and each of the eight 32nd notes
%   before and after (That's 8 + 1 + 8 = 17 elements * # of timbres)
%
% If the optional flag flg is given and flg==1, then the matrix entries
% are velocities instead of just binary values.


if nargin < 3
    flg = 0;
end


[dummy, timbres] = timbre_name(0, 0);
T = length(timbres);

B = 17; % length of detected beat vector

if (size(nm) ~= size(nmq))
    error('nm and nmq must be the same size!');
end
[M,N] = size(nm);


% Make timbre-specific nmq's to speed searching
tnmq = {};
for t=1:T,
    tnmq{t} = nmq(find(nmq(:,4) == timbres(t)), :);
end


% Make Y
Y = nm(:,1) - nmq(:,1);

% Make X
X = zeros(M, B*T);


for i=1:M,
    beat = nmq(i,1);
    beats = [beat-1 : 1/8 : beat+1];
    for tInd = 1:T,
        t = timbres(tInd);

        thisNmq = tnmq{tInd};
        timbreCompare = (thisNmq(:,4) == t);

        bInd = 1;
        for b = beats,
            onsetInd = find(and(thisNmq(:,1)==b, timbreCompare));
            if ~isempty(onsetInd)
                % beats are placed locally
                if (flg)
                    X(i, tInd + (bInd-1)*T) = thisNmq(onsetInd(1),5);
                else
                    X(i, tInd + (bInd-1)*T) = 1;
                end
            end

            bInd = bInd + 1;
        end
    end
    
    % Carry out rotation of beats to hopefully help increase eigenvalue spread
    % Each beat is a vector of length T
    rotBy = mod(beat*8, 32) * T;
    if (rotBy > B*T)
        rotBy = rotBy - B*T;    % <- because we have 4 beats per bar, but our 
    end                         % representation only handles 2 beats
    X(i,:) = [X(i,B*T-rotBy+1:B*T) X(i,1:B*T-rotBy)];
    
    if not(mod(i,100))
        disp([num2str(round(100*i/M)) '% done ...'])
    end
end
