function ind_cell = partition_dataset(nmq, k, beatmargin, nmq_usable_indices)
% Take all of the notes not within 'beatmargin' beats of the
% beginning or end of nmq, and randomly partition them into
% k groups.
%
% E.g., ind_cell = partition_dataset(escola_nmq, 5, 1)

ind_cell = cell(1, k);

nmq_firstbeat = min(nmq(:,1));
nmq_lastbeat = max(nmq(:,1));

if (nargin < 4)
    nmq_usable_indices = find(and(nmq(:,1) >= nmq_firstbeat+beatmargin, ...
        nmq(:,1) <= nmq_lastbeat-beatmargin));
end

size(nmq(:,4))

timbres = unique(nmq(:,4))

for t = timbres'
    ind = nmq_usable_indices(find(nmq(nmq_usable_indices, 4) == t));
    n = length(ind);

    disp(['timbre ' num2str(t) ' has ' num2str(n) ' notes.']);

    if n<k
        disp('Too few notes - dropping');
    end

    remainder = mod(n, k);
    if remainder == 0
        zeropad = 0;
    else
        zeropad = k-remainder;
    end

    randindices = [ind(randperm(n));  zeros(zeropad,1)];

    iM = reshape(randindices, k, length(randindices)/k);

    for i=1:k
        iMcol = iM(i,:);
        ind_cell{i} = [ind_cell{i} iMcol(find(iMcol~=0))];
    end
end
