if 0
  cd Documents/cs229/
  matlab -nojvm

  setpathccrma
  load_data
end

dataset = 'Brazil';
rhythmNumber = 1;
s = 3.1748e+03;
lambda = 0.4154;

officialPartitions = 1;
randomNotes = 0;
write_midi_sequences=1;

do_KNN_instead = 0;
k = 16;
tau = 24;

if do_KNN_instead
  midifilename = 'escola-knn-out.MID';
else
  midifilename = 'escola-krr-out.MID';
end


global xnmq ynmq ynm
if dataset == 'Brazil'
  rhythmnames = brazilrhythmnames;
  xnmq = nmq_cell{rhythmNumber};
  ynmq = nmq_cell{rhythmNumber};
  ynm = nm_cell{rhythmNumber};
else
  rhythmnames = reggaerhythmnames;
  xnmq = nmq_reg_cell{rhythmNumber};
  ynmq = nmq_reg_cell{rhythmNumber};
  ynm = nm_reg_cell{rhythmNumber};
end

rhythmname = rhythmnames{rhythmNumber}

distMeasure = 'notedistance';
  
DMfname = ['~/Documents/cs229/ourdata/' dataset '/Distances/' ...
	   rhythmname '-' distMeasure '.mat'];
disp(['Accessing the file ' DMfname])
load(DMfname)


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Set up training data and test data

if officialPartitions
  if dataset == 'Brazil'
    ind_cell = official_partitions{rhythmNumber};
  elseif dataset == 'Reggae'
    ind_cell = official_partitions_reggae{rhythmNumber};
  else
    error('Unrecognized dataset: no official partition');
  end
  
  k = length(ind_cell);
  
  xind_cell = cell(k,1);
  yind_cell = cell(k,1);
  
  for i = 1:k
    test = ind_cell{i};
    train = [];
    for j = 1:k
      if i~=j
	train = [train ind_cell{j}];
      end
    end
    xind_cell{i} = test;
    yind_cell{i} = train;
  end

else % not officialPartitions
  totalN = length(ynm);
  % totalN = 80;
  
  %% "Y" = training data
  %% "X" = new input
  
  xind = [];
  yind = [];
  
  disp('Throwing away first and last 20 notes because distance metric has')
  disp('issues with boundary conditions');
  usableIndices = 20:totalN-20;
  
  if randomNotes
    disp('Choosing random notes');
    for n=usableIndices
      if (rand > 0.7) 
	xind = [xind; n];
      else
	yind = [yind; n];
      end
    end
  else
    disp('Use last 70% as training data, rest as test data');
    cutoff = round(0.30 * length(usableIndices));
    xind = usableIndices(1:cutoff)';
    yind = usableIndices(cutoff+1:length(usableIndices))';
  end
  xind_cell = {xind};
  yind_cell = {yind};
end
  
output = ones(length(xnmq), 1) * -99;

num_trials = length(xind_cell);
for trial = 1:num_trials
  xind = xind_cell{trial};
  yind = yind_cell{trial};
  
  disp(['Training on ' num2str(length(yind)) ' notes, testing on ' ...
	num2str(length(xind)) ' notes.']);
  
  train = yind;
  test = xind;
  Ytest = ynm(test,1) - ynmq(test,1);
  true_devs = Ytest;

  tic
  if do_KNN_instead
    the_xnm = apply_kNN(test, train, k, tau, DM);
    YEst = the_xnm(test) - xnmq(test);
  else
    YEst = KRR(test, train, DM, 0, s, lambda);
  end
  elapsedtime = toc

  output(test) = YEst;
end

ind = find(output > -99);
true = ynm(ind)-ynmq(ind);
predicted = output(ind);

stem(true, 'b');
hold on
stem(predicted, 'r');

if do_KNN_instead
  algorithm = 'KNN';
else
  algorithm = 'KRR';
end


title(['Comparison of true microtimings vs. ' algorithm]);
ylabel('Microtiming (beats)');
xlabel('Note index (sequential)');
legend('True microtimings', ['Output of ' algorithm]);

if write_midi_sequences
  nmout = xnmq;
  nmout(ind, 1) = nmout(ind, 1) + predicted;
  writemidi(nmout, midifilename, 10000, gettempo(nmout));
  disp(['Wrote midi sequence ' midifilename])
end
