function [nmout matches distances errors] = simpleSynthesis(dataset, whichRhythm, whichTimb)
% KNN



if nargin<2
   whichRhythm = 'ESCOLA';
end

if nargin<1
   dataset = 'Brazil';
end

doplot = 1;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Load quantized/unquantized data
genre = dataset;
load_data;

if whichRhythm == 'OLODUM'
   nm = nm_cell{2};
   nmq = nmq_cell{2};
elseif whichRhythm == 'ESCOLA'
   nm = escola_nm;
   nmq = escola_nmq;
else
   error('Unknown rhythm');
end
disp('loaded quantized and unquantized sequences.');


% To test nearest-neighbor for only one particular timbre
if nargin >= 3
    timbInd = find(nm(:,4) == whichTimb);
    nm = nm(timbInd,:);
    nmq = nmq(timbInd,:);
end



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Set up training data and test data
totalN = length(nm);
% totalN = 80;

%% "Y" = training data
%% "X" = new input

xind = [];
yind = [];


disp('Throwing away first and last 20 notes because distance metric has')
disp('issues with boundary conditions');
usableIndices = 20:totalN-30;

randomNotes = 0;

if randomNotes
  disp('Choosing random notes');
  for n=usableIndices
     if (rand > 0.7) 
         xind = [xind; n];
     else
         yind = [yind; n];
     end
  end
else
     disp('Use last 70% as training data, rest as test data');
     cutoff = round(0.30 * length(usableIndices));
     xind = usableIndices(1:cutoff)';
     yind = usableIndices(cutoff+1:length(usableIndices))';
end

disp(['Training on ' num2str(length(yind)) ' notes, testing on ' num2str(length(xind)) ...
      ' notes.'])


global xnmq ynmq ynm;
% For this kind of test the test data and training data come from the same sequence
xnmq = nmq;
ynmq = nmq;
ynm = nm;


% The expensive part
k = 1;
[xnm matches distances] = apply_kNN(xind, yind, k);

% Check out how well we did
output_devs = xnm(xind, 1) - xnmq(xind, 1)
true_devs = ynm(xind, 1) - ynmq(xind, 1)


errors = (true_devs - output_devs);


% Put training data and output of our algorithm together into a sequence we can play
% Start with a copy of the original sequence
nmout = ynm;
% Now replace all the test set times with the output of our algorithm
nmout(xind) = xnm(xind);

%% Output some statistics about how well we did
errVar = sum(errors.^2)/length(errors)        % Variance of the error
mtVar = sum(true_devs.^2)/length(true_devs)   % Variance of actual microtimings

absErrAvg = sum(abs(errors))/length(errors)
absMtAvg = sum(abs(true_devs))/length(true_devs)



outputFilename = 'nearest-neighbor-test.MID';

writemidi(nmout(usableIndices,:), outputFilename, 10000, gettempo(nmout));
disp(['wrote file ' outputFilename]);

if doplot
disp('plotting...');
figure(1)
pianoroll(nm)
pianoroll(nmq,'hold','b')
pianoroll(nmout,'hold','g')
legend('Actual','Quantized','Algorithm Output')


figure(3)
hold off
stem(nm(xind,1),true_devs)
hold on
stem(nm(xind,1),errors,'r')
legend('Actual Microtimings','Errors',3)
title('Errors');

figure(4)
hold off
stem(nm(xind,1),true_devs.^2)
hold on
stem(nm(xind,1),errors.^2,'r')
legend('Actual Microtimings','Errors',3)
title('Squared Errors') 

disp('... done plotting.');
end 
