% script testgp
% 
% This is where we actually call the GP code from.

function [trainErr, testErr, wts] = testgp(tstPct, wts)

load_data
beatLocs = beatLocations3;
[dummy,BL] = size(beatLocs)


if (0)   % display number of onsets, etc.
    for i=1:9,
        nmq = nmq_cell{i};   % olodum for now on HH
        nm = nm_cell{i};     % (Rhythm 5 has no HH.)

        try
            [X,Y] = make_X(nm, nmq, 40);    % 42 is HH, 35 is kick
            [num2str(i) ' is of size ' num2str(length(Y)) ' and with tempo ' num2str(gettempo(nm))]
        catch
            lasterr;
        end
    end
end
X = [];
Y = [];
for i=[1],    % was [1 8 9]
    nmq = nmq_cell{i};   
    nm = nm_cell{i};

    % use one or the other
    [Xtmp,Ytmp] = make_X(nm, nmq, 70, beatLocs);    % 42 is HH, 35 is kick
    %[Xtmp,Ytmp,BL] = make_X_moredims(nm, nmq, 42);    % 42 is HH, 35 is kick
    
    X = [X; Xtmp];
    Y = [Y; Ytmp];
end
[M,N] = size(X);

% Test on 30% of the data if not otherwise specified
if (nargin < 1)
    tstPct = 0.30;
end

randData = 0;
if (randData)
    randInd = randperm(length(X));
    m = round(length(X)*(1-tstPct));

    Xtrain = X(randInd(1:m),:);
    Xtest = X(randInd(m+1:length(X)),:);

    Ytrain = Y(randInd(1:m),:);
    Ytest = Y(randInd(m+1:length(X)),:);
else
    sw = round(length(X)*tstPct);
    Xtest = X(1:sw,:);
    Xtrain = X(sw+1:length(X),:);

    Ytest = Y(1:sw,:);
    Ytrain = Y(sw+1:length(X),:);
end


% try subtracting out the mean ... could be important
ms = mean(Xtrain);
Xtrain = Xtrain - ones(length(Xtrain),1)*ms;
Xtest = Xtest - ones(length(Xtest),1)*ms;


W = N + 2;
if (nargin < 2) % need to estimate the weights
    prevWts = [];
    totStarts = 1;  % It seems to always converge to similar solutions, so just use one
    wts = zeros(W,totStarts);
    its = zeros(totStarts,1);
    fvals = zeros(totStarts,1);
    for i=1:totStarts,
        guesswts = rand(W,1) + 0.1*ones(W,1);
        guesswts(1:BL) = ones(BL,1)*4;

        if (length(prevWts) == 0)
            [wtsout, fWts, itsout] = minimize(guesswts, 'gp01lik', 100, Xtrain, Ytrain);
            %[wtsout, fWts, itsout] = minimize(guesswts, 'gp01lik', 70, X - ones(length(X),1)*ms, Y);
        else
            [wtsout, fWts, itsout] = minimize(prevWts, 'gp01lik', 70, Xtrain, Ytrain);
        end

        wts(:,i) = wtsout;
        fvals(i) = min(fWts);      % minimal values achieved by the optimization
        itsout(i) = itsout;
    end
    disp('exp(converged weights) was:')
    exp(wts)
end

% Try Gaussian Processes
[mu, S2] = gp01pred(wts, Xtrain, Ytrain, Xtest);

% Check the training error
[trainMu, S2] = gp01pred(wts, Xtrain, Ytrain, Xtrain);
trainErr = var(Ytrain-trainMu);


% Comparison with nearest neighbor (don't even plot this one it's so bad)
N = length(Ytest);
M = length(Ytrain);
nn = zeros(N,1);
for n=1:N,
    dist = inf;
    ind = 0;
    for m=1:M,
        cmpDist = norm(Xtest(n,:) - Xtrain(m,:));
        if (cmpDist < dist)
            dist = cmpDist;
            ind = m;
        end
    end
    nn(n) = Ytrain(ind);
end


% Comparison with scaled nearest neighbor
N = length(Ytest);
M = length(Ytrain);
snn = zeros(N,1);
swts = sqrt(exp(wts(1:W-2)))';
for n=1:N,
    dist = inf;
    ind = 0;
    for m=1:M,
        cmpDist = norm(swts .* (Xtest(n,:) - Xtrain(m,:)));
        if (cmpDist < dist)
            dist = cmpDist;
            ind = m;
        end
    end
    snn(n) = Ytrain(ind);
end

var(mu)
var(Ytest)
testErr = var(Ytest-mu);
var(Ytest-mu)
var(Ytest-nn)
var(Ytest-snn)


figure(1)
hold off
stem(Ytest)
hold on
stem(mu,'r')
stem(nn,'g')
stem(snn,'k-.')
legend('Actual','GP','NN','SNN')


figure(2)
hold off
stem(Ytest)
hold on
stem(mu,'r')
legend('Actual','GP')


