function testio(dataset)
% Tests the I/O functions of the midi toolbox to check that the 
% output is almost the same as the input.

[nm,name] = dir2coll(strcat(dataset,'/Original'));
outdir = strcat(dataset,'/CopiesOfOriginal/');


L = length(nm);
for l = 1:L,
    % Regarding tpq (ticks per quarter note)
    %
    % Cycle Ticks: Shows the number of MIDI ticks per cycle.
    % Number of ticks can be any value in valid range (1 - 65535).
    % Pull-down list values represent common note durations. Take
    % the number of cycle ticks divided by Ticks Per Quarter Note
    % (TPQ, see System Config) to determine the length of the
    % cycle in quarter notes.

    % inf and 50000 are too large, however,
    % 5000 and 10000 seem to be fairly accurate
    % (timing errors were on the order of 10^-5 sec when read
    % back into MATLAB)
    %
    % Small values however, led to quantization and a decreased tempo
     
    %strInd = findstr(name(l,:),'.MID');
    disp(sprintf('Using tempo %f for %s (%d notes)',gettempo(nm{l}),name(l,:),length(nm{l})))
    writemidi(nm{l},strcat(outdir,name(l,:)),10000,gettempo(nm{l}));
    
    % Check that the onset times are still correct (6th column)
    if (1)
        nmcmp = readmidi(strcat(outdir,name(l,:)));
        nmcmp = nmcmp - nm{l};
        maxOnsetErr = max(nmcmp(6,:));
        disp(sprintf('Maximum onset error: %f sec',maxOnsetErr))
    end
end
