function [T,notes] = timbre_distance_matrix()
% Compute T based on Matt's ad hoc opinions about timbral similarity
% "notes" is a list of notes that are counted in the matrix.

% Note numbers that appear in both ESCOLA.MID and OLODUM.MID
notes = [36 40 42 43 45 47 48 49 57 62 63 64 65 66 67 68 70 87 ];

% Default distance for unknown or erroneously unspecified timbres is zero:
T = zeros(max(notes));


% Construct upper-diagonal distance matrix

% Default distance between known timbres is 1
numnotes = length(notes);
for i = 1:numnotes
    T(notes(i), notes([i:numnotes])) = 1;
end

% Each timbre's distance to itself is zero
for n=notes
  T(n,n) = 0;
end

% Put in all "sort of sounds like" opinions row-by-row

% 36 = kick
T(36, 43) = 0.6;
T(36, [45 47]) = 0.8;
T(36, 62) = 0.9;
T(36, 63) = 0.85;
T(36, 64) = 0.7;
T(36, 87) = 0.2;

% 40 = snare
T(40, 42) = 0.5;
T(40, 43) = 0.95;
T(40, [45 47]) = 0.8;
T(40, [62:64]) = 0.9;
T(40, 65) = 0.7;

% 42 = closed HH
T(42, 70) = 0.6;

% 43 = tom low
T(43, [45 47]) = 0.2;
T(43, 48) = 0.4;
T(43, 63) = 0.65;
T(43, 64) = 0.4;
T(43, 87) = 0.5;

% 45, 47 = tom mid
T([45 47], 48) = 0.2;
T([45 47], [62:64]) = 0.5;
T([45 47], 66) = 0.666;
T([45 47], 87) = 0.7;

% 48 = tom hi
T(48, 62) = 0.3;
T(48, 63) = 0.2;
T(48, 64) = 0.5;
T(48, 66) = 0.8;
T(48, 87) = 0.95;

% 49 = crash
T(49, 57) = 0.1;

% 57 = crash mute
% Not like anything

% 62 = conga hi mute
T(62, 63) = 0.1;
T(62, 64) = 0.3;
T(62, 65) = 0.97;
T(62, 66) = 0.85;

% 63 = conga hi
T(63, 64) = 0.25;
T(63, 65) = 0.96;
T(63, 66) = 0.85;

% 65 = repique
T(65, 66) = 0.3;

% 67 = bell high
T(67, 68) = 0.1;
T(67, 70) = 0.9;


% disp('Here are the distances for notes I know about:')
% Tsmall = T(notes, notes)

% Convert to symmetric:
T = T + T';
